
#The source code is provided as a supplementary file for the paper called 'The Deriving Animal Movement Behaviors Using Movement Parameters Extracted from Location Data' 
#Corresponding auther: Maryam Teimouri

#We suggest users to upload their tracking dataset in movebank, an open portal of animal tracking data. 
#Then, they would not need to worry about the specific attribute names or order in their dataset to use our code

rm(list=ls()) #Removing all objects from the environment


#The required R packges
require(move) #For downloading dataset from movebank
require(bcpa) #Segmentation with bcpa package
require(circular) #For Circular statistics calculation
require(utils) #For getting input from user with syntax 'winDialogString' 


Userwindowsize<-30 #Windowsize can be changed based on the user consideration. We refer user to the paper
numBehaviours<-3 #The number of clusters can be changed based on visual inspectation of dendrogram, Persistence velocity boxplots and Turning angle histograms. We refer user to the paper


###########################################################
#Downloading animal tracking dataset
###########################################################

options(digits=10)
cat("Please Enter your movebank credentials");
cred <- movebankLogin()
StudyName<-readline(prompt='Please Enter your study name in movebank:')
AnimalID<-readline(prompt='Please Enter your Animal ID in movebank:') 
SheepDataset <- getMovebankData(study=StudyName,animalName=AnimalID, login=cred) 
ProjectionName<-readline(prompt='Please Enter EPSG identifier of your coordinate reference system , for example 32632 for UTM 32N')
mydata <- spTransform(SheepDataset, CRSobj=CRS(paste0('+init=epsg:',ProjectionName)))#Project the dataset from the default long/lat coordinate to your projection coordinate system
mydata$x<-mydata@coords[,1]
mydata$y<-mydata@coords[,2]


dateorderlist<-unique(as.Date(mydata$timestamp))
cat("All available dates in your dataset:",as.character(dateorderlist)) #it Shows all unique dates in your datset and user needs to select one date 
mydate<-winDialogString("Please select your date?",toString(dateorderlist[1]))
mydatadaynth=mydata[which(as.Date(mydata$timestamp)==mydate),]
mydatadaynth<-as.data.frame(mydatadaynth)



###########################################################
#Preparing dataset for bcpa package
###########################################################
mymat<-mydatadaynth[,c('x','y','timestamp')]
colnames(mymat)[1]<-'X'
colnames(mymat)[2]<-'Y'
colnames(mymat)[3]<-'Time'

###########################################################
#Segmentation with bcpa package
###########################################################

mymat.VT <- GetVT(mymat,units='min')
par(mfrow=c(1,1))

hist(mymat.VT$V, breaks=20, col="grey")
hist(mymat.VT$Theta, breaks=20, col="grey")

mymat.ws <- WindowSweep(mymat.VT, "V*cos(Theta)", windowsize=Userwindowsize, progress=TRUE,  K=2,tau=FALSE)
mymat2.ws <- WindowSweep(mymat.VT, "V*sin(Theta)", windowsize=Userwindowsize, progress=TRUE, K=2,tau=FALSE)

DiagPlot(mymat.ws)
DiagPlot(mymat2.ws)

myresult<-ChangePointSummary(mymat.ws,clusterwidth =1)
myresult2<-ChangePointSummary(mymat2.ws,clusterwidth= 1)


#we segment the animal movement path twice using BCPA (first for Persistence velocity and then for Turning velocity)
#we combine and order the change points and remove duplicates

FirstCos<-data.frame(myresult$breaks)[c(2,5)]
SecondSin<-data.frame(myresult2$breaks)[c(2,5)]
names(SecondSin)<-names(FirstCos)
wholeChangePoints<-rbind(FirstCos,SecondSin)
SortedwholeChangePoints<-wholeChangePoints[order(wholeChangePoints$middle.POSIX),]
IndextobeDeleted<-which(diff(SortedwholeChangePoints$middle.POSIX)==0)
MyFinalResult<-SortedwholeChangePoints[-IndextobeDeleted,]
TotalSeg<-nrow(MyFinalResult)

###########################################################
#Extracting time intervals for all segments
###########################################################

MySegTime1<-c(toString(mydatadaynth$timestamp[1]),toString(MyFinalResult$middle.POSIX[1]) )
for (i in 2:(TotalSeg-1))
{
  temptime<-c(toString(MyFinalResult$middle.POSIX[i-1]),toString(MyFinalResult$middle.POSIX[i]))
  assign(paste('MySegTime',i,sep=''),temptime)
}


temptime<-c(toString(MyFinalResult$middle.POSIX[TotalSeg-1]) ,toString(mydatadaynth$timestamp[nrow(mydatadaynth)]))
assign(paste('MySegTime',TotalSeg,sep=''),temptime)

###########################################################
# Getting all observations including turning angles, speeds and Persistence velocities separatelly for each segement  
###########################################################


temp<-mymat.VT[1:(max(which(mymat.VT$T.mid<=MyFinalResult$middle[1]))),]

Newtemp<-temp$V*cos(temp$Theta)
myvec1<-Newtemp
MyTurn1<-temp$Theta
MySpeed1<-temp$V


for (i in 2:(TotalSeg-1))
  
{
  temp<-mymat.VT[(max(which(mymat.VT$T.mid<=MyFinalResult$middle[i-1]))+1):(max(which(mymat.VT$T.mid<=MyFinalResult$middle[i]))),]
  Newtemp<-temp$V*cos(temp$Theta)
  assign(paste('myvec',i,sep=''),Newtemp)
  
  MyTurntemp<-temp$Theta
  assign(paste('MyTurn',i,sep=''),MyTurntemp)
  MySpeedtemp<-temp$V
  assign(paste('MySpeed',i,sep=''),MySpeedtemp)
}


temp<-mymat.VT[(max(which(mymat.VT$T.mid<=MyFinalResult$middle[TotalSeg-1]))+1):nrow(mymat.VT),]
Newtemp<-temp$V*cos(temp$Theta)


assign(paste('myvec',TotalSeg,sep=''),Newtemp)
MyTurntemp<-temp$Theta
assign(paste('MyTurn',TotalSeg,sep=''),MyTurntemp)
MySpeedtemp<-temp$V
assign(paste('MySpeed',TotalSeg,sep=''),MySpeedtemp)

###########################################################
#calculating dissimilarity(ks_distance) for every pair of segments and making distance matrix and Dendrogram
###########################################################
result <- vector("list", TotalSeg) 
for (i in 1:TotalSeg)
{result[[i]]<-eval(parse(text=paste('myvec',i,sep='') ))}


myksdistance<-matrix(nrow=TotalSeg,ncol=TotalSeg)
for (i in 1:TotalSeg){
  for (j in 1:TotalSeg){ if (i==j){myksdistance[i,j]=0 } else {
    temp<-ks.test(eval(parse(text=paste('myvec',i,sep='') )),eval(parse(text=paste('myvec',j,sep='') )))
    myksdistance[i,j]<-(temp$statistic)}}}


DistanceMatrix<-as.dist(myksdistance)
mytree<-hclust(DistanceMatrix, method='ward.D2' )
plot(hclust(DistanceMatrix,method='ward.D2'),cex=0.7,cex.lab=1.5,cex.main=1.5, cex.sub=1.5,hang = -1)


myindex<-cutree(mytree,k=numBehaviours)
rect.hclust(mytree, k = numBehaviours, border = 2:4) 

###########################################################
#To know the number of segments in each behaviour
###########################################################

for (r in 1:numBehaviours)
{
  temp<-which(myindex==r)
  #print(temp)
  print(length(temp))
  
}


###########################################################
#Investigating each behaviour using Persistence velocity: v*cos(theta) 
###########################################################

OutputPlot <- vector("list", numBehaviours) 
for (r in 1:numBehaviours)
 {
    temp<-which(myindex==r)
    myspeedtemp<-numeric()
  
    for (i in temp)
    {
      myspeedtemp<-append(myspeedtemp,mean(eval(parse(text=paste('myvec',i,sep='') ))))
    }
  OutputPlot[[r]]<-myspeedtemp
  }
boxplot(OutputPlot, outline=FALSE)

###########################################################
#Investigating each behaviour using mean of turning angles
###########################################################

#calculate the mean of turning angles for each segemnt and then plot the means

for (r in 1:numBehaviours)
{
  temp<-which(myindex==r)
  myangtemp<-numeric()
  for (i in temp)
  {
    tempangle<-circular(eval(parse(text=paste('MyTurn',i,sep=''))),type='angles', units="radians",zero = circular(0),modulo='2pi',rotation = "counter")
    myangtemp<-append(myangtemp,mean.circular(tempangle))
   }
  MyFinalTurn<-myangtemp
  plot(MyFinalTurn,stack=TRUE)
  arrows.circular(mean(MyFinalTurn), y=rho.circular(MyFinalTurn), lwd=1)
  
}

###########################################################
#Output in text format to show all segments in each behaviour
###########################################################
for (r in 1:numBehaviours)
  
  
{
  temp<-which(myindex==r)
  MyBehaviour<-character()
  
  
  for (i in 1:length(temp)){j<-temp[i]
  MyBehaviour<-append(MyBehaviour,eval(parse(text=paste('MySegTime',j,sep='') )))}
  
  myvalues<-as.data.frame(table(MyBehaviour))
  myrepetirion<-as.character(myvalues$MyBehaviour[which(myvalues$Freq==2)])
  MyBehaviour<-MyBehaviour[!MyBehaviour %in% myrepetirion]
  MyBehaviour<-matrix(MyBehaviour, ncol=2, byrow=TRUE)
  colnames(MyBehaviour)<-c("start time","End time")
  assign(paste('Behaviour',r,sep=''),MyBehaviour)
  cat("Behaviour", r)
  print(eval(parse(text=paste('Behaviour',r,sep='') )))
  
}
